import javafx.application.Application;
import javafx.scene.text.Font;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.KeyCode;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonType;

/**
 * コントロールクラス
 * <pre>
 * 当クラスで全体的な制御を行います。
 * </pre>
 */
public class KCtrlSKK extends KCtrl implements KConstant {

	/** フェーズ */
	public static final int			PHASE_INIT			= 0;
	public static final int			PHASE_TITLE			= 1;
	public static final int			PHASE_START			= 2;
	public static final int			PHASE_RUN			= 3;
	public static final int			PHASE_STAGE_CLEAR	= 4;
	public static final int			PHASE_BOMB			= 5;
	public static final int			PHASE_END			= 6;
	public static final int			PHASE_STOP			= -1;

	/** パネル */
	private		KPanel				panel				= null;

	/**
	 * スプライト
	 * <pre>
	 * 全体的に表示等されるスプライトは、ここで管理します。
	 * </pre>
	 */
	private		KSpriteBackground	spriteBackground0	= null;
	private		KSpriteBackground	spriteBackground1	= null;
	private		KSpriteTitle		spriteTitle			= null;
	private		KSpriteShip			spriteShip			= null;
	private		KSpriteShipMini		spriteShipMini		= null;

	/** テキスト */
	private		KText				textStart			= null;
	private		KText				textSetumei			= null;
	private		KText				textBy				= null;
	private		KText				textEnd				= null;
	private		KText				textScoreHigh		= null;
	private		KText				textScore			= null;
	private		KTextShield			textShield			= null;
	private		KText				textStage			= null;
	private		KText				textTime			= null;
	private		KText				textBonus			= null;

	/** キャンバス */
	private		KCanvasTitleFX		canvasTitleFX			= null;

	/** フェーズ */
	private		int					phase				= PHASE_INIT;

	/** ストーリー */
	private		KStorySKK			storySKK			= null;

	/** スコア */
	private		int					score				= 0;

	/** ハイスコア */
	private		int					scoreHigh			= 50000;

	/** ステージ */
	private		int					stage				= 1;

	/** 時間（ワーク用） */
	private		int					timeWork			= 0;

	/** 防御 */
	private		int					shield				= 5;
	/** 防御 */
	private		String				strShield			= "■■■■■";

	/** Qキー押下フラグ */
	private		boolean				flgQ				= false;

	/** Pause制御フラグ */
	private		boolean				flgPause			= false;

	/** 撃墜してからの時間 */
	private		int					timeBomb			= 0;

	/** BGM */
	private		KSoundMP3[]			mp3Stages			= null;
	private		KSoundMP3			mp3Stage			= null;
	private		KSoundMP3[]			mp3Bosss			= null;
	private		KSoundMP3			mp3Boss				= null;

	/** 効果音 */
	private		KSoundWave			waveTitle			= null;
	private		KSoundWave			waveFX				= null;
	private		KSoundWave			waveCredit			= null;
	private		KSoundWave[]		waveShots			= null;
	private		KSoundWave[]		waveTekiTaerus		= null;
	private		KSoundWave[]		waveTekiBombs		= null;
	private		KSoundWave[]		waveJimenTaerus		= null;
	private		KSoundWave[]		waveJimenBombs		= null;
	private		KSoundWave[]		waveT08Imbibe		= null;
	private		KSoundWave			waveCollision		= null;
	private		KSoundWave			waveBomb			= null;
	private		KSoundWave			waveItemGet			= null;
	private		KSoundWave			waveAppearSYATIHOKO	= null;
	private		KSoundWave			waveBombSYATIHOKO	= null;
	private		KSoundWave			waveT31Laser		= null;
	private		KSoundWave			waveT34Laser		= null;
	private		KSoundWave			waveBossWarning		= null;
	private		KSoundWave			waveStageClear		= null;
	private		KSoundWave			waveAllClear		= null;
	private		KSoundWave			wavePinch			= null;
	private		KSoundWave			waveGameOver		= null;

	/** ビーム音制御 */
	private		int					posWaveShot			= 0;
	/** 敵耐える制御 */
	private		int					posWaveTekiTaeru	= 0;
	/** 敵爆発音制御 */
	private		int					posWaveTekiBomb		= 0;
	/** 地面耐える制御 */
	private		int					posWaveJimenTaeru	= 0;
	/** 地面爆発音制御 */
	private		int					posWaveJimenBomb	= 0;
	/** T08吸い込み制御 */
	private		int					posWaveT08Imbibe	= 0;

	/** シャチホコ出現フラグ */
	private		boolean				flgBombSYATIHOKO	= false;

	/**
	 * コンストラクタ
	 * @param con コンテナ
	 */
	public KCtrlSKK(Application application){

		super(application);

		try{

			// パネルを生成
			panel = new KPanel(this);

			// 伸縮モードを設定
			panel.setStretchScreen(true);

			// スプライト層の数を設定
			panel.setLayoutCount(LAYOUT_COUNT);

			// ストーリークラスを生成
			storySKK = new KStorySKK(panel);

			// イメージをスプライトに登録する
			KSpriteBackground.initImage();
			KSpriteTitle.initImage();
			KCanvasTitleFX.initImage();
			KSpriteShip.initImage();
			KSpriteShipMini.initImage();
			KSpriteBeam1.initImage();
			KSpriteS01.initImage();
			KSpriteS02.initImage();
			KSpriteS03.initImage();
			KSpriteS04.initImage();
			KSpriteT00.initImage();
			KSpriteT01.initImage();
			KSpriteT02.initImage();
			KSpriteT03.initImage();
			KSpriteNikomi.initImage();
			KSpriteT04.initImage();
			KSpriteT05.initImage();
			KSpriteT06.initImage();
			KSpriteT07.initImage();
			KSpriteT08.initImage();
			KSpriteT09.initImage();
			KSpriteT11.initImage();
			KSpriteT12.initImage();
			KSpriteT13.initImage();
			KSpriteT31.initImage();
			KSpriteT32.initImage();
			KSpriteT33.initImage();
			KSpriteT34.initImage();
			KSpriteT42.initImage();
			KSpriteT43.initImage();
			KSpriteT99.initImage();
			KSpriteT9A.initImage();
			KSpriteT9B.initImage();
			KSpriteJ01.initImage();
			KSpriteJ02.initImage();
			KSpriteJ03.initImage();
			KSpriteJ04.initImage();
			KSpriteJ31.initImage();
			KSpriteJ41.initImage();
			KSpriteJ42.initImage();
			KSpriteJ80.initImage();
			KSpriteJ81.initImage();
			KSpriteG01.initImage();
			KSpriteG11.initImage();
			KSpriteG31.initImage();
			KSpriteG37.initImage();
			KSpriteG38.initImage();
			KSpriteG41.initImage();
			KSpriteG51.initImage();
			KSpriteG61.initImage();
			KSpriteG62.initImage();
			KSpriteG64.initImage();
			KSpriteG71.initImage();

			// スプライト生成
			spriteBackground0 = new KSpriteBackground(panel);
			spriteBackground1 = new KSpriteBackground(panel);
			spriteTitle = new KSpriteTitle(panel);
			spriteShip = new KSpriteShip(panel);
			spriteShipMini = new KSpriteShipMini(panel);

			// 背景を登録
			panel.addSprite(LAYOUT_BACKGROUND, spriteBackground0);
			panel.addSprite(LAYOUT_BACKGROUND, spriteBackground1);

			// 各テキストを生成
			textStart = new KText(panel, 215, 400);
			textStart.setFont(Font.font(null, 16));
			textStart.setText("PUSH SPACE KEY");

			textSetumei = new KText(panel, 144, 460);
			textSetumei.setFont(Font.font(null, 12));
			textSetumei.setText("Q - PAUSE    A - BEAM ON / OFF    Z - BEAM SHOT");

			textBy = new KText(panel, 168, 520);
			textBy.setFont(Font.font(null, 12));
			textBy.setText("PROGRAMED  BY  KIN  /  MUSIC  BY  Aki2");

			textEnd = new KText(panel, 250, 300);
			textEnd.setFont(Font.font(null, 16));
			textEnd.setText("GAME OVER");

			textScoreHigh = new KText(panel, 50, 20);
			textScoreHigh.setFont(Font.font(null, 12));
			panel.addText(LAYOUT_TEXT, textScoreHigh);

			textScore = new KText(panel, 200, 20);
			textScore.setFont(Font.font(null, 12));
			panel.addText(LAYOUT_TEXT, textScore);

			textShield = new KTextShield(panel, 350, 20);
			textShield.setFont(Font.font(null, 12));
			panel.addText(LAYOUT_TEXT, textShield);

			textStage = new KText(panel, 500, 20);
			textStage.setFont(Font.font(null, 12));
			panel.addText(LAYOUT_TEXT, textStage);

			textTime = new KText(panel, 500, 40);
			textTime.setFont(Font.font(null, 12));
			panel.addText(LAYOUT_TEXT, textTime);

			textBonus = new KText(panel, 200, 300);
			textBonus.setFont(Font.font(null, 16));
			textBonus.setText("SYATIHOKO BONUS 100,000");

			// 各キャンバスを生成
			canvasTitleFX = new KCanvasTitleFX(panel);

			// 残機スプライトの設定
			spriteShipMini.setLayoutX(320);
			spriteShipMini.setLayoutY(4);

			// 背景に切れ目ができないように設定
			spriteBackground1.setLayoutY(-600);

			// 効果音を生成
			waveTitle = new KSoundWave(this, "sound\\SKK_Title.wav", false);
			waveFX = new KSoundWave(this, "sound\\SKK_FX.wav", false);
			waveCredit = new KSoundWave(this, "sound\\SKK_Credit.wav", false);

			waveShots = new KSoundWave[10];
			for(int i = 0; i < 10; i++){
				waveShots[i] = new KSoundWave(this, "sound\\SKK_Shot.wav", false);
			}

			waveTekiTaerus = new KSoundWave[8];
			for(int i = 0; i < 8; i++){
				waveTekiTaerus[i] = new KSoundWave(this, "sound\\SKK_TekiTaeru.wav", false);
			}
			waveTekiBombs = new KSoundWave[8];
			for(int i = 0; i < 8; i++){
				waveTekiBombs[i] = new KSoundWave(this, "sound\\SKK_TekiBomb.wav", false);
			}
			waveJimenTaerus = new KSoundWave[8];
			for(int i = 0; i < 8; i++){
				waveJimenTaerus[i] = new KSoundWave(this, "sound\\SKK_JimenTaeru.wav", false);
			}
			waveJimenBombs = new KSoundWave[8];
			for(int i = 0; i < 8; i++){
				waveJimenBombs[i] = new KSoundWave(this, "sound\\SKK_JimenBomb.wav", false);
			}
			waveT08Imbibe = new KSoundWave[8];
			for(int i = 0; i < 8; i++){
				waveT08Imbibe[i] = new KSoundWave(this, "sound\\SKK_T08Imbibe.wav", false);
			}
			waveCollision = new KSoundWave(this, "sound\\SKK_Collision.wav", false);
			waveBomb = new KSoundWave(this, "sound\\SKK_Bomb.wav", false);
			waveItemGet = new KSoundWave(this, "sound\\SKK_ItemGet.wav", false);
			waveAppearSYATIHOKO = new KSoundWave(this, "sound\\SKK_AppearSYATIHOKO.wav", false);
			waveBombSYATIHOKO = new KSoundWave(this, "sound\\SKK_BombSYATIHOKO.wav", false);
			waveT31Laser = new KSoundWave(this, "sound\\SKK_T31Laser.wav", false);
			waveT34Laser = new KSoundWave(this, "sound\\SKK_T34Laser.wav", false);
			waveBossWarning = new KSoundWave(this, "sound\\SKK_BossWarning.wav", false);
			waveStageClear = new KSoundWave(this, "sound\\SKK_StageClear.wav", false);
			waveAllClear = new KSoundWave(this, "sound\\SKK_AllClear.wav", false);
			wavePinch = new KSoundWave(this, "sound\\SKK_Pinch.wav", true);
			waveGameOver = new KSoundWave(this, "sound\\SKK_GameOver.wav", false);

			// BGMを生成
			mp3Stages = new KSoundMP3[6];
			mp3Stages[0] = new KSoundMP3(this, "sound\\SKK_Stage01.mp3", true);
			mp3Stages[1] = new KSoundMP3(this, "sound\\SKK_Stage02.mp3", true);
			mp3Stages[2] = new KSoundMP3(this, "sound\\SKK_Stage03.mp3", true);
			mp3Stages[3] = new KSoundMP3(this, "sound\\SKK_Stage04.mp3", true);
			mp3Stages[4] = new KSoundMP3(this, "sound\\SKK_Stage05.mp3", true);
			mp3Stages[5] = new KSoundMP3(this, "sound\\SKK_Stage06.mp3", true);
			mp3Bosss = new KSoundMP3[1];
			mp3Bosss[0] = new KSoundMP3(this, "sound\\SKK_StageBoss_01.mp3", true);

			// メディアビューを追加
			for(int i = 0; i < mp3Stages.length; i++){
				panel.addMediaView(LAYOUT_MEDIA, mp3Stages[i]);
			}

		// エラーの場合、
		}catch(Exception ex){
			// スタックトレース出力
			ex.printStackTrace();
			// エラーを表示
			Alert alert = new Alert(AlertType.ERROR , "" , ButtonType.OK);
			alert.setTitle("エラー");
			alert.getDialogPane().setContentText(ex.toString());
			alert.showAndWait();

		}

	} // end KCtrlSKK

	/**
	 * 開始
	 * <pre>
	 * 当オブジェクトの処理を開始します。
	 * <pre>
	 */
	public void start(){

		// パネルにフォーカスを当てる
		panel.requestFocus();

		// パネルを開始する
		panel.start();

		// 初期化フェーズへ
		KCtrlSKK.this.phase = PHASE_INIT;

	} // end start

	/**
	 * 実行
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * </pre>
	 * @return  true：正常 ／ false：一時停止
	 */
	public boolean run(){

		// フェーズ分け
		switch(this.phase){

			// 初期化フェーズ
			case PHASE_INIT:

				// BGMを初期化
				for(int i = 0; i < mp3Stages.length; i++){
					mp3Stages[i].init();
				}
				for(int i = 0; i < mp3Bosss.length; i++){
					mp3Bosss[i].init();
				}

				// スコア
				score = 0;

				// ステージ
				stage = 1;

				// 防御
				shield = 5;
				strShield = "■■■■■";

				// 撃墜時間
				timeBomb = 0;

				// シャチホコ出現フラグ
				flgBombSYATIHOKO = false;

				// ストーリをリセット
				storySKK.reset();

				// 自機のスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_JIBUN);

				// 星のスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_STAR);

				// 地面のスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_JIMEN_0);
				panel.initKSpriteLstInLayout(LAYOUT_JIMEN_1);

				// 敵のスプライト層を初期化
				panel.initKSpriteLstInLayout(LAYOUT_TEKI);

				// スプライトグループ初期化
				panel.initSpriteGroup();

				// 背景を初期化
				spriteBackground0.setImage(0);
				spriteBackground1.setImage(0);

				// キャンバスの時間を戻す
				canvasTitleFX.init();

				// タイトル表示
				panel.addSprite(LAYOUT_JIMEN_0, spriteTitle);
				panel.addText(LAYOUT_TEXT, textStart);
				panel.addText(LAYOUT_TEXT, textBy);
				panel.addText(LAYOUT_TEXT, textSetumei);
				panel.addCanvas(LAYOUT_JIMEN_1, canvasTitleFX);

				// スコア初期化
				textScore.setText("SCORE : " + Integer.toString(score));

				// ハイスコア設定
				textScoreHigh.setText("HIGH-SCORE : " + Integer.toString(scoreHigh));

				// 効果音をスタート
				waveTitle.start();

				// タイトル画面フェーズへ
				this.phase = PHASE_TITLE;

				break;

			// タイトル画面フェーズ
			case PHASE_TITLE:

				// スペースが押された場合、
				if(panel.isKeyPressed(KeyCode.SPACE)){

					// タイトルを削除
					panel.removeSprite(LAYOUT_JIMEN_0, spriteTitle);
					panel.removeText(LAYOUT_TEXT, textStart);
					panel.removeText(LAYOUT_TEXT, textBy);
					panel.removeText(LAYOUT_TEXT, textSetumei);
					panel.removeCanvas(LAYOUT_JIMEN_1, canvasTitleFX);

					// 時間を初期化
					timeWork = 0;

					// 効果音をスタート
					waveCredit.start();

					// スタートフェーズへ
					this.phase = PHASE_START;

				}

				break;

			// スタートフェーズ
			case PHASE_START:

				// 時間＋１
				timeWork++;

				// 1.5秒経ったら、
				if(timeWork > 90){

					// 自機を生成
					spriteShip = new KSpriteShip(panel);

					// 自機を初期化
					spriteShip.init();

					// 自機スプライトを登録
					panel.addSprite(LAYOUT_JIBUN, spriteShip);
					// 残機スプライトを登録
					panel.addSprite(LAYOUT_JIBUN, spriteShipMini);

					// ストーリーを進める
					storySKK.start();

					// BGM取得
					mp3Stage = mp3Stages[stage - 1];

					// BGMをスタート
					mp3Stage.start();

					// 通常フェーズへ
					this.phase = PHASE_RUN;

				}

				break;

			// 通常フェーズ
			case PHASE_RUN:

				// Qキーを押していなかった時に押した時、
				if(panel.isKeyPressed(KeyCode.Q) && !flgQ){
					// ポーズ制御
					this.flgPause = !this.flgPause;
				}

				// Qキー制御
				flgQ = panel.isKeyPressed(KeyCode.Q);

				// ポーズの場合、
				if(flgPause){
					// 一時停止でリターン
					return false;
				}

				//
				// ストーリーを進める
				//
				storySKK.next();

				// 各テキストを最新化
				textScoreHigh.setText("HIGH-SCORE : " + Integer.toString(scoreHigh));

				textScore.setText("SCORE : " + Integer.toString(score));

				textTime.setText("TIME : " + storySKK.getStoryTime());

				textStage.setText("STAGE : " + Integer.toString(stage));

				textShield.setText(strShield);

				// シャチホコ出現中の場合、
				if(flgBombSYATIHOKO){

					// 時間＋１
					timeWork++;

					// 4秒
					if(timeWork >= 540){

						// シャチホコ終わり
						flgBombSYATIHOKO = false;

						// ボーナス削除
						panel.removeText(LAYOUT_TEXT, textBonus);

						// BGMスタート
						mp3Stage.start();

					}

				} // end if シャチホコ出現中の場合

				break;

			// 撃墜フェーズ
			case PHASE_BOMB:

				//
				// ストーリーを進める
				//
				storySKK.next();

				// 各テキストを最新化
				textTime.setText("TIME : " + storySKK.getStoryTime());

				textShield.setText(strShield);

				// 撃墜時間を進める
				timeBomb++;

				// 時間が経った場合、
				if(timeBomb > 210){

					// 効果音をスタート
					waveGameOver.start();

					// ゲームオーバーフェーズへ
					phase = PHASE_END;

					// ゲームオーバーを表示
					panel.addText(LAYOUT_TEXT, textEnd);

				}

				break;

			// ゲームオーバーフェーズ
			case PHASE_END:

				//
				// ストーリーを進める
				//
				storySKK.next();

				// 各テキストを最新化
				textTime.setText("TIME : " + storySKK.getStoryTime());

				// 撃墜時間を進める
				timeBomb++;

				// 時間が経った場合、
				if(timeBomb > 960){

					// ゲームオーバーを削除
					panel.removeText(LAYOUT_TEXT, textEnd);

					// GC
					this.gc(true);

					// 初期化フェーズへ
					phase = PHASE_INIT;

				}

				break;

			default:
				break;

		} // end switch フェーズ分け

		// 正常でリターン
		return true;

	} // end run

	/**
	 * スコア加算
	 * <pre>
	 * スコア加算する場合は、呼び出してください。
	 * </pre>
	 * @param scoreAdd 追加する点数
	 */
	public void addScore(int scoreAdd){

		// スコアを加算
		this.score = this.score + scoreAdd;

		// ハイスコアを更新する
		this.scoreHigh = (this.score > this.scoreHigh)?this.score: this.scoreHigh;

		// ハイスコアを最新化
		textScoreHigh.setText("HIGH-SCORE : " + Integer.toString(scoreHigh));

		// スコアを最新化
		textScore.setText("SCORE : " + Integer.toString(score));

	} // end addScore

	/**
	 * スコア取得
	 * @return int スコア
	 */
	public int getScore(){

		return this.score;

	} // end getScore

	/**
	 * 防御加減算
	 * <pre>
	 * プラスもマイナスも処理します。
	 * 0になったら、ゲームオーバーです。
	 * </pre>
	 * @param shieldAdd 加減算する数
	 * @return 加減算した後の防御数
	 */
	public int addShield(int shieldAdd){

		// 防御加減算
		this.shield = this.shield + shieldAdd;

		StringBuilder sb = new StringBuilder();

		// 防御テキストを作成しなおす
		for(int i = 0; i < this.shield; i++){
			sb.append("■");
		}

		strShield = sb.toString();

		// ダメージを受けた場合（受けた影響が−１以下の場合）、
		if(shieldAdd < 0){
			// 色を変更する
			textShield.damage();
		}

		// 防御が１になった場合、
		if(this.shield == 1){

			// ピンチ音をスタート
			wavePinch.start();

		// 防御が１から２になった場合、
		}else if(this.shield == 2 && shieldAdd == 1){

			// ピンチ音をストップ
			wavePinch.stop();

		// 防御が０になった場合、
		}else if(this.shield == 0){

			// ピンチ音をストップ
			wavePinch.stop();

			// BGMをストップ
			mp3Stage.stop();
			if(mp3Boss != null){
				mp3Boss.stop();
			}

			// 撃墜フェーズへ
			phase = PHASE_BOMB;

		}

		// 加減算した後の防御数をリターン
		return this.shield;

	} // end addScore

	/**
	 * 防御取得
	 * <pre>
	 * </pre>
	 * @return int 防御数
	 */
	public int getShield(){

		return this.shield;

	} // end getShield

	/**
	 * フェーズ再開
	 */
	public void restart(){

		// ゲームオーバー中の場合、
		if(this.phase == PHASE_BOMB || this.phase == PHASE_END){

			// 何もしない

		// 上記以外の場合、
		}else{

			// フェーズ再開
			this.phase = PHASE_RUN;

		}

	}

	/**
	 * フェーズ取得
	 */
	public int getPhase(){

		return phase;

	}

	/**
	 * ステージ取得
	 */
	public int getStage(){

		// ステージをリターン
		return this.stage;

	}

	/**
	 * ステージ＋１
	 */
	public void addStage(){

		// ステージを＋１
		this.stage++;

		// GC
		this.gc(false);

		// BGM取得
		mp3Stage = mp3Stages[stage - 1];

		// BGM演奏位置初期化
		mp3Stage.init();
		// BGMをスタート
		mp3Stage.start();

		// STAGE4以外
		if(stage != 4){

			// 背景を設定
			spriteBackground0.setImage(0);
			spriteBackground1.setImage(0);

		// STAGE4
		}else{

			// 背景を設定
			spriteBackground0.setImage(1);
			spriteBackground1.setImage(1);

		} // end if

	} // end addStage

	/**
	 * パネル取得
	 * @return KPanel パネル
	 */
	public KPanel getKPanel(){

		// パネルをリターン
		return this.panel;

	}

	/**
	 * ストーリークラス取得
	 * @return KPanel パネル
	 */
	public KStory getKStory(){

		// パネルをリターン
		return this.storySKK;

	}

	/**
	 * 自機取得
	 * @return KSpriteShip 自機
	 */
	public KSpriteShip getSpriteShip(){
		// 自機取得
		return spriteShip;
	}

	/**
	 * ビキーンスタート
	 */
	public void startWaveFX(){

		// 効果音をスタート
		waveFX.start();

	}

	/**
	 * ビーム音スタート
	 */
	public void startWaveBeam(){

		// ビーム音制御
		posWaveShot = (posWaveShot == 9)?0: ++posWaveShot;

		// 効果音をスタート
		waveShots[posWaveShot].start();

	}

	/**
	 * 敵耐える音スタート
	 */
	public void startWaveTekiTaeru(){

		// 敵爆発音音制御
		posWaveTekiTaeru = (posWaveTekiTaeru == 7)?0: ++posWaveTekiTaeru;

		// 効果音をスタート
		waveTekiTaerus[posWaveTekiTaeru].start();

	}

	/**
	 * 敵爆発音スタート
	 */
	public void startWaveTekiBomb(){

		// 敵爆発音音制御
		posWaveTekiBomb = (posWaveTekiBomb == 7)?0: ++posWaveTekiBomb;

		// 効果音をスタート
		waveTekiBombs[posWaveTekiBomb].start();

	}

	/**
	 * 地上物耐える音スタート
	 */
	public void startWaveJimenTaeru(){

		// 地上物爆発音音制御
		posWaveJimenTaeru = (posWaveJimenTaeru == 7)?0: ++posWaveJimenTaeru;

		// 効果音をスタート
		waveJimenTaerus[posWaveJimenTaeru].start();

	}

	/**
	 * 地上物爆発音スタート
	 */
	public void startWaveJimenBomb(){

		// 地上物爆発音音制御
		posWaveJimenBomb = (posWaveJimenBomb == 7)?0: ++posWaveJimenBomb;

		// 効果音をスタート
		waveJimenBombs[posWaveJimenBomb].start();

	}

	/**
	 * T08吸い込み音スタート
	 */
	public void startWaveT08Imbibe(){

		// T08吸い込み音制御
		posWaveT08Imbibe = (posWaveT08Imbibe == 7)?0: ++posWaveT08Imbibe;

		// 効果音をスタート
		waveT08Imbibe[posWaveT08Imbibe].start();

	}

	/**
	 * 敵接触音スタート
	 */
	public void startWaveCollision(){
		// 効果音をスタート
		waveCollision.start();
	}

	/**
	 * 爆発音スタート
	 */
	public void startWaveBomb(){
		// 効果音をスタート
		waveBomb.start();
	}

	/**
	 * ステージ音スタート
	 */
	public void startMp3Stage(){

		// BGMをストップ
		mp3Stage.start();

	}

	/**
	 * ステージ音ポーズ
	 */
	public void pauseMp3Stage(){

		// BGMをストップ
		mp3Stage.pause();

	}

	/**
	 * ステージ音ストップ
	 */
	public void stopMp3Stage(){

		// BGMをストップ
		mp3Stage.stop();

	}

	/**
	 * ボス音スタート
	 * @param track トラック
	 */
	public void startMp3Boss(int track){

		// BGMを取得
		mp3Boss = mp3Bosss[track];

		// BGMをスタート
		mp3Boss.start();

	}

	/**
	 * ボス音ストップ
	 */
	public void stopMp3Boss(){

		// BGMオブジェクトが存在する場合、
		if(mp3Boss != null){

			// BGMをストップ
			mp3Boss.stop();

			// BGMを初期化
			// （ここで初期化しておく）
			mp3Boss.init();

		} // end if

	} // stopMp3Boss

	/**
	 * アイテム取得スタート
	 */
	public void startWaveItemGet(){
		// 効果音をスタート
		waveItemGet.start();
	}

	/**
	 * シャチホコ出現
	 */
	public void appearSYATIHOKO(){
		// 効果音をスタート
		waveAppearSYATIHOKO.start();
	}

	/**
	 * シャチホコ破壊
	 */
	public void bombSYATIHOKO(){

		// シャチホコ破壊
		flgBombSYATIHOKO = true;

		// ボーナス表示
		panel.addText(LAYOUT_TEXT, textBonus);

		// 時間を初期化
		timeWork = 0;

		// BGMをポーズ
		mp3Stage.pause();

		// 効果音をスタート
		waveBombSYATIHOKO.start();

	} // end bombSYATIHOKO

	/**
	 * レーザーT31
	 */
	public void laserT31(){
		// 効果音をスタート
		waveT31Laser.start();
	}

	/**
	 * レーザーT34
	 */
	public void laserT34(){
		// 効果音をスタート
		waveT34Laser.start();
	}

	/**
	 * ボス警告
	 */
	public void startBossWarning(){
		// 効果音をスタート
		waveBossWarning.start();
	}

	/**
	 * ステージクリア
	 */
	public void stageClear(){

		// BGMをストップ
		mp3Stage.stop();

		// BGM（ボス）をストップ
		// 存在している場合のみ、ストップ
		stopMp3Boss();

		// 効果音をスタート
		waveStageClear.start();

		// ステージクリアフェーズ
		this.phase = PHASE_STAGE_CLEAR;

	}

	/**
	 * オールクリア
	 */
	public void allClear(){

		// 効果音をストップ
		waveStageClear.stop();
		// 効果音をスタート
		waveAllClear.start();

		// フェーズ変更
		phase = PHASE_END;

	}

	/**
	 * ゲームオーバー
	 */
	public void gameover(){

		// BGMをストップ
		stopMp3Stage();
		stopMp3Boss();

		// 効果音をスタート
		waveGameOver.start();

		// フェーズ変更
		phase = PHASE_END;

		panel.addText(LAYOUT_TEXT, textEnd);

	} // end gameover

	/**
	 * GC
	 * <pre>
	 * GCと、メモリの状態をコンソールに出力します。
	 * </pre>
	 * @param flagGC true：GCを行う ／ false：GCを行わない
	 */
	private void gc(boolean flagGC){

		// GCを行う場合、
		if(flagGC){
			// GC
			System.gc();
		}

		Runtime runtime = Runtime.getRuntime();
		System.out.println("MAX -" + runtime.maxMemory() / 1000 + "KBytes");
		System.out.println("USED-" + (runtime.totalMemory() - runtime.freeMemory()) / 1000 + "KBytes");
	}

}
