import java.util.HashMap;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;

/**
 * スプライト
 * <pre>
 * スプライトオブジェクト。
 * ImageViewを継承し、イメージを表示しますが、
 * 拡張し、途中でイメージを変更できるようにします。
 * </pre>
 */
abstract public class KSprite extends ImageView {

	/** パネル */
	protected KPanel panel = null;

	/** イメージ・クラス設定 */
	private static HashMap<Class, Image[]>imageThiss = new HashMap<Class, Image[]>();

	/** イメージ現在設定 */
	private Image imageThis = null;

	/** 幅（基本） */
	protected double baseWidth = -1;

	/** 高さ（基本） */
	protected double baseHeight= -1;

	/** 表示しているクラス */
	protected Class theClass = null;

	/**
	 * コンストラクタ
	 * @param panel パネル
	 */
	public KSprite(KPanel panel){

		super();

		// パネルを保持
		this.panel = panel;

		// クラスを取得
		this.theClass = this.getClass();

		// イメージが保持されている場合、
		if(imageThiss.get(this.getClass()) != null){

			// イメージ設定
			this.imageThis = imageThiss.get(this.getClass())[0];
			super.setImage(this.imageThis);

			// 幅（基本）設定
			this.baseWidth = this.imageThis.getWidth();

			// 高さ（基本）設定
			this.baseHeight = this.imageThis.getHeight();

			// 境界に合わせて幅・高さを設定
			super.setFitWidth(this.baseWidth);
			super.setFitHeight(this.baseHeight);

		} // end if イメージが保持されている場合

	} // end KSprite

	/**
	 * イメージ保持
	 * <pre>
	 * イメージを保持します。毎回、イメージを呼び出すのは重い可能性があるため、保持します。
	 * staticメソッドです。
	 * </pre>
	 * @param theClass スプライトクラス
	 * @param image イメージ
	 */
	public static void setImage(Class theClass, Image image){
		KSprite.setImage(theClass, new Image[]{image});
	}

	/**
	 * イメージ保持（複数）
	 * <pre>
	 * イメージを保持します。毎回、イメージを呼び出すのは重い可能性があるため、保持します。
	 * staticメソッドです。
	 * </pre>
	 * @param theClass スプライトクラス
	 * @param images イメージ（複数）
	 */
	public static void setImage(Class theClass, Image[] images){
		KSprite.imageThiss.put(theClass, images);
	}

	/**
	 * 実行
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * 継承先で定義をしてください。
	 * </pre>
	 */
	abstract public void run();

	/**
	 * イメージ設定
	 * @imagePhase イメージフェーズ
	 */
	public void setImage(int imagePhase){
		// イメージ設定
		this.imageThis = imageThiss.get(this.getClass())[imagePhase];
		// イメージビューに設定
		super.setImage(this.imageThis);
	}

	/**
	 * イメージ設定
	 * @theClass スプライトクラス 自クラスと違うスプライトを指定する場合は、このメソッドを呼び出す
	 * @imagePhase イメージフェーズ
	 */
	public void setImage(Class theClass, int imagePhase){
		// クラス保持
		this.theClass = theClass;
		// イメージ設定
		this.imageThis = imageThiss.get(theClass)[imagePhase];
		// イメージビューに設定
		super.setImage(this.imageThis);
	}

	/**
	 * 幅取得
	 * @return double 幅
	 */
	public double getWidth(){
		return imageThiss.get(theClass)[0].getWidth();
	}

	/**
	 * 高さ取得
	 * @return double 高さ
	 */
	public double getHeight(){
		return imageThiss.get(theClass)[0].getHeight();
	}

	/**
	 * 幅（基本）取得
	 * @return double 幅（基本）
	 */
	public double getBaseWidth(){
		return this.baseWidth;
	}

	/**
	 * 幅（基本）設定
	 * @param x 幅（基本）
	 */
	public void setBaseWidth(double x){
		this.baseWidth = x;
	}

	/**
	 * 高さ（基本）取得
	 * @return double 高さ（基本）
	 */
	public double getBaseHeight(){
		return this.baseHeight;
	}

	/**
	 * 高さ（基本）設定
	 * @param x 高さ（基本）
	 */
	public void setBaseHeight(double y){
		this.baseHeight = y;
	}

	/**
	 * スプライト削除
	 * <pre>
	 * 自スプライトを削除します。
	 * <pre>
	 */
	public void removeSprite(){

		// スプライトを削除
		panel.removeSprite(this);

	} // end removeSprite

	/**
	 * パネル取得
	 * @return KPanel パネル
	 */
	public KPanel getKPanel(){
		return panel;
	}

}
