import java.util.*;
import javafx.scene.image.Image;
import javafx.scene.Node;

/**
 * 敵０５ KK-GR リボ
 */
public class KSpriteT05 extends KSpriteT00 implements KConstant {

	//
	// 敵をいい感じで移動させるために、いろいろと計算している
	//
	private	final	int			SPRITE_WIDTH = 32;

	private final	int			MARGIN = 48;

	private final	int			RANGE = (WIDTH - MARGIN * 2) / 2;

	private final	int			CENTER = WIDTH / 2 - SPRITE_WIDTH;

	private			boolean		flgCourse = false;

	/** ビーム発射位置 */
	private			int			timeBeam	= -1;

	/**
	 * イメージ初期化
	 * <pre>
	 * イメージの取り込みを行います。
	 * </pre>
	 */
	public static void initImage() {

		Image imageT05_00 = null;
		Image imageT05_01 = null;
		Image imageT05_02 = null;
		Image imageT05_03 = null;

		imageT05_00 = KImage.getImage("image\\imageT05_00.gif");
		imageT05_01 = KImage.getImage("image\\imageT05_01.gif");
		imageT05_02 = KImage.getImage("image\\imageT05_02.gif");
		imageT05_03 = KImage.getImage("image\\imageT05_03.gif");

		KSpriteT05.setImage(KSpriteT05.class, new Image[]{imageT05_00, imageT05_01, imageT05_02, imageT05_03});

	} // end initImage

	/**
	 * コンストラクタ
	 * @param panel KPanel
	 * @param storyParam ビーム発射位置
	 */
	public KSpriteT05(KPanel panel, String storyParam){

		super(panel);

		// 自機を取得
		KSpriteShip spriteShip = ((KCtrlSKK)panel.getKCtrl()).getSpriteShip();

		// 自機が左側にいる場合、
		if(spriteShip.getLayoutX() < WIDTH / 2){
			// コース変更
			flgCourse = true;
		}

		super.setLayoutY(-getHeight());

		// バグ対応
		// 設定しないと、一番左でビームを出すと、撃墜できる時がある。
		super.setLayoutX(-100);

		// ビーム発射位置が設定されている場合、
		if(storyParam != null && !storyParam.equals("-1")){
			// ビーム発射位置を保存
			timeBeam = Integer.parseInt(storyParam);
		}

	} // end KSpriteT05

	/**
	 * 実行
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * 継承先で定義をしてください。
	 * <pre>
	 */
	public void run(){

		// 親クラスを実行する
		super.run();

		double x = super.getLayoutX();
		double y = super.getLayoutY();

		// 前進
		y = y + 1;

		// 画面の外に出た場合、
		if(y > HEIGHT){
			// スプライトを消す
			super.removeSprite();

		}

		// X方向の移動
		// Y方向に、円の移動量を計算してX方向の位置を計算
		// もっと簡単な計算方法があるかもしれない。疑似的な計算でもいいと思う。
		// ようはうまく動けばいいと思う。
		x = CENTER + Math.sin(Math.toRadians(((y * 2 + (flgCourse?180: 0)) % 360))) * RANGE;

		super.setLayoutX(x);
		super.setLayoutY(y);

		// 生きている場合、
		if(!isBomb()){

			// 一定量進んだ場合、
			if(y % 6 == 0){
				// イメージを変更する
				// くるくる回ってる感じを出す
				setImage(KSpriteT05.class, (int)(Math.abs(y / 6) % 4));
			}

			// ビームを発射する場合、
			if(timeBeam != -1 && y == timeBeam){

				// ビーム発射
				KSpriteT99 spriteT99 = new KSpriteT99(super.getKPanel(), this, 6);
				super.getKPanel().addSprite(LAYOUT_TEKI, spriteT99);

			}

		} // end if 生きている場合

	} // end run

	/**
	 * 爆発
	 */
	public void bomb(){

		// 100点
		((KCtrlSKK)panel.getKCtrl()).addScore(100);

		super.bomb();

	} // end bomb

}
